//=============================================================================
// (c) Copyright 2005 Diamond Systems Corporation. Use of this source code
// is subject to the terms of Diamond Systems' Software License Agreement.
// Diamond Systems provides no warranty of proper performance if this
// source code is modified.
//
// File: DSCSetUserInt.c   v5.9
// Desc: Sample program that demonstrates how to setup User Int with other 
//       Interrupt Operations
// Created by  KL
//=============================================================================

#include <stdio.h>

#ifdef _WIN32
#ifndef _WIN32_WCE
#include <conio.h>
#endif

#include <windows.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
// diamond driver includes
#include "dscud.h"
#endif

// DOS
#ifdef __BORLANDC__
#include <conio.h>
#include <dos.h>
#include <stdlib.h>
#include <math.h>
#include <mem.h>
// diamond driver includes
#include "../../../../current/dev/source/dscud.h"
#endif

// Linux and QNX
#if defined(linux) || defined(__QNXNTO__) || defined(_WRS_VXWORKS_5_X)
#include <stdlib.h>
#include <math.h>
#include <time.h>
#include <string.h>
// diamond driver includes
#include "dscud.h"


#ifdef _WRS_VXWORKS_5_X
#include <selectLib.h>
#define main AthenaDSCSetUserInt
#else
#include <sys/time.h>
#endif

static int kbhit()
{
	struct timeval timeout;
	fd_set rfds;

	timeout.tv_sec = 0;
	timeout.tv_usec = 0;

	FD_ZERO(&rfds);
	FD_SET(0, &rfds);

	if ( select(0+1, &rfds, NULL, NULL, &timeout) > 0 )
		return getchar();

	return 0;
}
#endif

// macros defined
#define SLEEP_TIME 1000

// var declarations
BYTE result; // returned error code
DSCB dscb;   // handle used to refer to the board
DSCCB dsccb; // structure containing board settings
DSCS dscs;   // used for interrupts.
DSCADSETTINGS dscadsettings; // structure containing A/D conversion settings
DSCAIOINT dscaioint;         // structure containing auto-calibration settings
DFLOAT voltage;
ERRPARAMS errorParams;  // structure for returning error code and error string
int intBuff;            // temp variable of size int
long longBuff;          // temp variable of size long
float floatBuff;        // temp variable of size float
DSCUSERINTFUNCTION dscuserintfunc; // structure for setting up user int
int counter;	// the global variable to be modified by the user defined function
int i;	     // miscellaneous counter

void count(void * param);

//=============================================================================
// Name: main()
// Desc: Starting function that calls the driver functions used
//
//		 NOTE: By convention, you should capture the BYTE return value for each
//		 driver API call, and check the error code.
//
//	     I. Driver Initialization
//	    II. Board Initialization
//	   III. User Settings
//	    IV. AD Settings
//	     V. IO Settings and UserInt settings
//		VI. User Function
//	   VII. Cleanup
//
//=============================================================================

int main( void )
{
	//=========================================================================
	// I. DRIVER INITIALIZATION
	//
	//    Initializes the DSCUD library.
	//
	//=========================================================================

	if( dscInit( DSC_VERSION ) != DE_NONE )
	{
		dscGetLastError(&errorParams);
		fprintf( stderr, "dscInit error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
		return 0;
	}

	//=========================================================================
	// II. BOARD INITIALIZATION
	//
	//	   Initialize the ATHENA board. This function passes the various
	//	   hardware parameters to the driver and resets the hardware.
	//
	//     STEPS TO FOLLOW:
	//
	//	   1. set the board type to DSC_ATHENA for ATHENA board
	//	   2. set the base address to 0x280
	//	   3. set the interrupt level 5
	//	   4. intialize and register the board with the driver, after which
	//		  the struct, dscb, now holds the handle for the board
	//=========================================================================

	printf( "\nATHENA BOARD INITIALIZATION:\n" );

	dsccb.base_address = 0x280;

	dsccb.int_level = 5;

	if(dscInitBoard(DSC_ATHENA, &dsccb, &dscb)!= DE_NONE)
	{
		dscGetLastError(&errorParams);
		fprintf( stderr, "dscInitBoard error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
		return 0;
	}

	//===========================================================================
	// III. Setup User Interrupts
	//
	//		Specify the user interrupt function ( make sure you have a function 
	//		pointer already ) And Assign the mode to 
	//		USER_INT_AFTER or USER_INT_INSTEAD if you want to use the instead mode.
	//
	//===========================================================================

	//	DSCUserInterruptFunction func;
	//	BYTE int_mode;

	counter =0;
	dscuserintfunc.func = count; // user interrupt function pointer
	dscuserintfunc.int_mode = USER_INT_AFTER;   // user interrupt mode
	if( (result = dscSetUserInterruptFunction( dscb, &dscuserintfunc)) != DE_NONE)
	{
		dscGetLastError(&errorParams);
		fprintf( stderr, "dscSetUserInterruptFunction error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
		return 0;
	}

	//=========================================================================
	// IV. AD SETTINGS INITIALIZATION
	//
	//	    Initialize the structure containing the AD conversion settings and
	//		then pass it to the driver.
	//
	//=========================================================================

	/* PRE-FILLED EXAMPLE
	dscadsettings.range = RANGE_10;
	dscadsettings.polarity = BIPOLAR;
	dscadsettings.gain = GAIN_1;
	dscadsettings.load_cal = (BYTE)TRUE;
	dscadsettings.current_channel = 0;
	*/

	printf( "\nAD SETTINGS INITIALIZATION\n" );

	memset(&dscadsettings, 0, sizeof(DSCADSETTINGS));

	dscadsettings.range = RANGE_10;

	printf( "Enter the polarity (0 for BIPOLAR, 1 for UNIPOLAR): " );
	scanf("%d", &intBuff);
	dscadsettings.polarity = (BYTE) intBuff;

	printf( "Enter the gain (0 for GAIN 1, 1 for GAIN 2, 2 for GAIN 4, 3 for GAIN 8): " );
	scanf("%d", &intBuff);
	dscadsettings.gain = (BYTE) intBuff;

	dscadsettings.load_cal = 0;

	dscadsettings.current_channel = 0;

	if( ( result = dscADSetSettings( dscb, &dscadsettings ) ) != DE_NONE )
	{
		dscGetLastError(&errorParams);
		fprintf( stderr, "dscADSetSettings error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
		return 0;	}

	//=========================================================================
	// V. I/O INTERRUPT SETTINGS INITIALIZATION
	//
	//	   Initialize the structure containing the analog I/O interrupt
	//	   settings.
	//
	//	   NOTE: You must allocate space for the buffer holding the returned
	//		     sample values. Also, be generous in allocating storage.
	//			 Allocating insufficient memory to hold sample data will result
	//			 in improper behavior of the driver, such as hanging interrupt
	//			 operations or assertion errors.
	//
	//=========================================================================

	/* PRE-FILLED EXAMPLE
	dscaioint.num_conversions = 1024;
	dscaioint.conversion_rate = 1000;
	dscaioint.cycle = (BYTE)FALSE;
	dscaioint.internal_clock = (BYTE)TRUE;
	dscaioint.low_channel = 0;
	dscaioint.high_channel = 3;
	dscaioint.external_gate_enable = (BYTE)FALSE;
	dscaioint.internal_clock_gate = (BYTE)FALSE;
	dscaioint.fifo_enab = (BYTE)TRUE;
	dscaioint.fifo_depth = 32;
	dscaioint.dump_threshold = 256;
	*/

	memset(&dscaioint, 0, sizeof(DSCAIOINT));

	printf( "\nI/O INTERRUPT SETTINGS INITIALIZATION\n" );

	printf( "Enter the number of conversions (must be a multiple of FIFO depth) : " );
	scanf("%ld", &longBuff);
	dscaioint.num_conversions = (DWORD) longBuff;

	printf( "Enter the conversion rate in Hz (must be less than 100000) : " );
	scanf("%f", &floatBuff);
	dscaioint.conversion_rate = (FLOAT) floatBuff;

	printf( "Enter the cycle flag (0 for FALSE, 1 for TRUE) : " );
	scanf("%d", &intBuff);
	dscaioint.cycle = (BOOL) intBuff;

	printf( "Enter the internal clock flag (0 for FALSE, 1 for TRUE) : " );
	scanf("%d", &intBuff);
	dscaioint.internal_clock  = (BOOL) intBuff;

	printf( "Enter the lowest channel in the scan range (0-15) : " );
	scanf("%d", &intBuff);
	dscaioint.low_channel = (BYTE) intBuff;

	printf( "Enter the highest channel in the scan range (0-15) : " );
	scanf("%d", &intBuff);
	dscaioint.high_channel = (BYTE) intBuff;

	dscaioint.external_gate_enable = 0; // can enable it if need be

	dscaioint.internal_clock_gate = 0;   // can enable it if need be

	dscaioint.fifo_enab = 1;

	printf( "Enter the FIFO depth (must be 46 or less) : " );
	scanf("%d", &intBuff);
	dscaioint.fifo_depth = (WORD) intBuff;

	printf("Enter the dump threshold ( must be less or equal to num conversions ) : ");
	scanf("%ld", &longBuff);
	dscaioint.dump_threshold = (DWORD)longBuff;

	dscaioint.sample_values = (DSCSAMPLE*)malloc( sizeof(DSCSAMPLE) * dscaioint.num_conversions );

	getchar();

	//=========================================================================
	// VI. SAMPLING AND OUTPUT
	//
	//    Perform the actual sampling and then output the results. To calculate
	//	  the actual input voltages, we must convert the sample code (which
	//	  must be cast to a short to get the correct code) and then plug it
	//	  into one of the formulas located in the manual for your board (under
	//	  "A/D Conversion Formulas"). 
	//=========================================================================

	printf( "\nSAMPLING AND OUTPUT\n" );

	if( ( result = dscADSampleInt( dscb, &dscaioint ) ) != DE_NONE )
	{
		dscGetLastError(&errorParams);
		fprintf( stderr, "dscADSampleInt error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
		free( dscaioint.sample_values ); // remember to deallocate malloc() memory
		return 0;
	}

	// check interrupt status
	dscs.transfers = 0;
	dscs.overflows = 0;
	dscs.op_type = OP_TYPE_INT;

	do{
		dscSleep(SLEEP_TIME);
		dscGetStatus(dscb, &dscs);	
		printf("Fifo Overflows %lu , A/D Int Trans in progress: %lu  Total Trans %lu \n", dscs.overflows, dscs.transfers, dscs.total_transfers);
		printf("\nUserInt counter %d\n", counter);
	}while (dscs.op_type != OP_TYPE_NONE) ;

	// cancel interrupts manually for recycled mode or if interrupts are still running
	if( dscs.op_type != OP_TYPE_NONE)
	{
		if( (result = dscCancelOp(dscb)) != DE_NONE)
		{
			dscGetLastError(&errorParams);
			fprintf( stderr, "dscCancelOp error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
			free( dscaioint.sample_values ); // remember to deallocate malloc() memory
			return 0;
		}
	}
	
	
	printf( "\nSample readouts:" );

	for( i = dscaioint.low_channel; i < dscaioint.high_channel + 1; i++)
		printf( " %hd", dscaioint.sample_values[i] );

	printf( "\nActual voltages:" );

	for( i = dscaioint.low_channel; i < dscaioint.high_channel + 1; i++)
	{
		if( dscADCodeToVoltage(dscb, dscadsettings, dscaioint.sample_values[i], &voltage) != DE_NONE)
		{
			dscGetLastError(&errorParams);
			fprintf( stderr, "dscADCodeToVoltage error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
			free(dscaioint.sample_values);
			return 0;
		}

		printf( " %5.3lfV", voltage);
	}

	printf( "\n\n" );
	
	
	//=========================================================================
	// VII. CLEANUP
	//
	//	   Cleanup any remnants left by the program and free the resources used
	//	   by the driver.
	//
	//     STEPS TO FOLLOW:
	//
	//	   1. free the memory allocated for sample values
	//	   2. free the driver resources
	//=========================================================================

	free( dscaioint.sample_values );

	dscuserintfunc.func = NULL;
	dscuserintfunc.int_mode = USER_INT_CANCEL;
	if( (result = dscSetUserInterruptFunction(dscb, &dscuserintfunc)) != DE_NONE)
	{
		dscGetLastError(&errorParams);
		fprintf( stderr, "dscSetUserInterruptFunction error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
		return 0;
	}

	dscFree();

	printf( "\nDSCSetUserInt completed. \n" );

	return 0;
} // end main()



///////////////////////////
// User Interrupt Function
///////////////////////////

void count(void * param)
{
	counter ++;
}


#ifdef _WIN32_WCE
int WINAPI WinMain(	HINSTANCE hInstance,
					HINSTANCE hPrevInstance,
					LPTSTR    lpCmdLine,
					int       nCmdShow)
{
	main();

	return 0;
}

#endif
